
module FIFO 

//////////////////PARAMETERS///////////////////////////

#(
	parameter 
	DATA_SIZE_BITS = 16,
	DEPTH 			= 128
)

//////////////////INPUTS AND OUTPUTS////////////////////

(
	input 								  clk,
	input 								  rd_en,
	input									  wr_en, 
	output  								  full,
	output								  empty,
	output reg  [clogb2(DEPTH)-1:0] fifo_counter,
	input 	  [DATA_SIZE_BITS-1:0] fifo_in,
	output reg [DATA_SIZE_BITS-1:0] fifo_out,
	output reg 							  data_out_updated,
	input  								  rst_n
);

//////////////////REGISTERS/////////////////////////////

	reg 			[clogb2(DEPTH)-1:0] rd_ptr;
	reg 			[clogb2(DEPTH)-1:0] wr_ptr;
	reg  		  [DATA_SIZE_BITS-1:0] fifo_mem [DEPTH-1:0];
	
//////////////////COMBINATORIAL////////////////////////
	
	assign  full = (fifo_counter == DEPTH);
	assign empty = (fifo_counter == 0);
	
//////////////////SEQUENTIAL///////////////////////////	

	always@(posedge clk or negedge rst_n) begin
		if(!rst_n)
			fifo_counter = 0; 
		else if ((!full && wr_en)&&(!empty && rd_en))
			fifo_counter = fifo_counter;
		else if(!full && wr_en) 
			fifo_counter = fifo_counter+1;
		else if(!empty && rd_en)
			fifo_counter = fifo_counter-1;
	end


	always@(posedge clk or negedge rst_n) begin
		if(!rst_n) begin
			fifo_out <= 0;
			data_out_updated <=  0;
		end
		else begin
			data_out_updated <=  rd_en & !empty;
			if(!empty && rd_en) 
				fifo_out <= fifo_mem[rd_ptr];
		end
	end


	always@(posedge clk) begin
		if(!full && wr_en)
			fifo_mem[wr_ptr] <= fifo_in;
	end
	
	always@(posedge clk or negedge rst_n) begin
		if(!rst_n) begin
			wr_ptr <= 0;
			rd_ptr <= 0;
		end
		else begin
			if(!full && wr_en)
				wr_ptr <= wr_ptr +1;
			if(!empty && rd_en)
				rd_ptr <= rd_ptr+1;		
		end			
	end


//Function for calculating number of bits needed to cover DEPTH.

	function integer clogb2;
	input [31:0] depth;
		begin
			depth = depth-1;
			for(clogb2 = 0; depth>0; clogb2 = clogb2 +1)
				depth = depth >> 1;
		end
	endfunction
endmodule
