

module IMF_check 

	//////////////////PARAMETERS///////////////////////

#(
	parameter
	DATA_SIZE_BITS 		 = 16, 
	PRECISION_BITS_MEMORY = 8, 
	CHECKING_PRECISION 	 = 4
)

	//////////////////INPUTS AND OUTPUTS//////////////

(
	input 															clk,
	input [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in,
	input 															data_in_available,
	input 															first,
	input 															last,
	output reg 														already_IMF,
	input 															reset
);

	//////////////////REGISTERS///////////////////////

	reg 																already_IMF_temp;

	//////////////////WIRES///////////////////////////
	 
	wire [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0]  data_out_f_e;
	wire 																point_is_max;
	wire																point_is_min;
	wire																data_out_available_f_e;
	wire 																first_out_f_e;
	wire																last_out_f_e;

	//////////////////MODULES/////////////////////////

	find_extrema #(
						DATA_SIZE_BITS, 
						PRECISION_BITS_MEMORY, 
						CHECKING_PRECISION
					  )
						
						f_e
						
					  (
						clk, 
						data_in, 
						data_in_available, 
						first, 
						last,
						data_out_f_e, 
						point_is_max, 
						point_is_min, 
						data_out_available_f_e, 
						first_out_f_e, 
						last_out_f_e,reset
					  );

	//////////////////SEQUENTIAL///////////////////////////

	always @(posedge clk) begin

		if(reset) begin
			already_IMF_temp <= 0;
			already_IMF <= 0;
		end 
		else if(first_out_f_e) begin
			already_IMF_temp <= 1; 
		end
		else if (!last_out_f_e &&(((data_out_f_e[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1]==1) && point_is_max)	||	((data_out_f_e[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1]==0) && point_is_min))) begin
			already_IMF_temp <= 0;
		end 
		else if(last_out_f_e) begin
			already_IMF <= already_IMF_temp;
		end
		else begin
			already_IMF <= 0; 
		end
	end

endmodule 

