

module calculate_coefficients 

	//////////////////PARAMETERS///////////////////////////

#(
	parameter 
	DATA_SIZE_BITS 		 = 16,
	PRECISION_BITS_MEMORY = 8,
	DEPTH 					 = 512
)

	//////////////////INPUTS AND OUTPUTS////////////////////

(
	input 																 	 clk,
	input 																	 rst_n,
	input 																	 data_in_available,
	input 																	 first_in,
	input 																	 last_in,
	input signed [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] y_m,
	input signed [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] y,
	input signed [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] y_p,
	input 								  		  [clogb2(DEPTH)-1:0] h_m,
	input 								  		  [clogb2(DEPTH)-1:0] h,
	output reg 																 data_out_available,
	output reg 																 first_out,
	output reg 																 last_out,
	output reg 								 		 [clogb2(DEPTH):0] A,
	output reg 								 	  [clogb2(DEPTH)+2:0] B,
	output reg 								 		 [clogb2(DEPTH):0] C,
	output reg	[DATA_SIZE_BITS + PRECISION_BITS_MEMORY +3:0] D
);

	//////////////////WIRES////////////////////////////////

	wire signed 									 [clogb2(DEPTH):0] h_signed;
	wire signed 									 [clogb2(DEPTH):0] h_m_signed;

	//////////////////COMBINATORIAL////////////////////////
	
	assign h_signed	[clogb2(DEPTH)-1:0] = h;
	assign h_signed	    [clogb2(DEPTH)] = 0;

	assign h_m_signed [clogb2(DEPTH)-1:0] = h_m;
	assign h_m_signed 	 [clogb2(DEPTH)] = 0;

	//////////////////SEQUENTIAL///////////////////////////
	
	always @(posedge clk or negedge rst_n) begin
		if(!rst_n) begin
			A <= 0;
			B <= 0;
			C <= 0;
			D <= 0;

		end
		else if(data_in_available) begin				
			A <= h_m;													
			B <= (h_m + h)*2;						
			C <= h;														
			D <= ($signed(y_p-y)/h_signed - $signed(y-y_m)/h_m_signed)*2*3;
		end
	end


	always @(posedge clk, negedge rst_n) begin
		if(!rst_n) begin
			data_out_available <= 0;
			first_out <= 0;
			last_out <= 0;
		end
		else begin
			data_out_available <= data_in_available;
			first_out <= first_in;
			last_out <= last_in;
		end
	end


	//Function for calculating number of bits needed to cover DEPTH.

		function integer clogb2;
		input [31:0] depth;
			begin
				depth = depth-1;
				for(clogb2 = 0; depth>0; clogb2 = clogb2 +1)
					depth = depth >> 1;
			end
		endfunction

endmodule
