

module find_avg_spline 

	//////////////////PARAMETERS///////////////////////

#(
	parameter 
	DATA_SIZE_BITS 		 = 16, 
	PRECISION_BITS_CS 	 = 16, 
	PRECISION_BITS_MEMORY = 8, 
	CHECKING_PRECISION 	 = 4, 
	DEPTH 					 = 512
)

	//////////////////PARAMETERS///////////////////////

(
	input 															 clk,
	input  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in,
	input 															 data_in_available,
	input 															 first,
	input 															 last,
	output [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out,
	output 															 first_out_avg_spline,
	output 															 last_out_avg_spline,
	output 															 data_out_available,
	input 															 reset
);

	//////////////////WIRES///////////////////////////

	wire 																 already_IMF;
	wire 																 prev_already_IMF;
	wire 																 data_min_available;
	wire 																 data_max_available;
	wire 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_min;
	wire 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_max;
	wire 																 data_ready;
	wire 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] extremes;
	wire 																 point_is_max;
	wire																 point_is_min;
	wire																 first_out;
	wire																 start_point;
	wire																 end_point;
	wire																 last_out;
	wire																 first_out_max;
	wire																 first_out_min;
	wire																 last_out_max;
	wire 																 last_out_min;

	//////////////////MODULES/////////////////////////

	find_extrema #(
						DATA_SIZE_BITS, 
						PRECISION_BITS_MEMORY, 
						CHECKING_PRECISION
					  )
						
						find_extrema1
						
					  (
						clk,
						data_in,
						data_in_available,
						first,
						last,
						extremes,
						point_is_max,
						point_is_min,
						data_ready,
						first_out,
						last_out,
						reset
					  );

	cubic_spline #(
						DATA_SIZE_BITS, 
						PRECISION_BITS_CS, 
						PRECISION_BITS_MEMORY, 
						DEPTH
					  )
						
						CS_UPPER
						
					  (
						clk,
						!reset,
						point_is_max,
						data_ready,
						start_point,
						end_point,
						extremes,
						data_max,
						first_out_max,
						last_out_max,
						data_max_available
					  ); 

	cubic_spline #(
						DATA_SIZE_BITS, 
						PRECISION_BITS_CS, 
						PRECISION_BITS_MEMORY, 
						DEPTH
					  )
						
						CS_LOWER
						
					  (
						clk,
						!reset,
						point_is_min,
						data_ready,
						start_point,
						end_point,
						extremes,
						data_min,
						first_out_min,
						last_out_min,
						data_min_available
					  );


	collect_and_avg #(
							DATA_SIZE_BITS + PRECISION_BITS_MEMORY, 
							DEPTH
						  )
							
							collect_and_avg1
							
						  (
							clk,
							data_max,
							data_max_available,
							data_min,
							data_min_available,
							first_out_min,
							last_out_min,
							data_out,
							first_out_avg_spline,
							last_out_avg_spline,
							data_out_available,
							reset
						  );

	//////////////////COMBINATORIAL///////////////////						
							
	assign start_point = first_out;
	assign end_point 	 = last_out;
	
endmodule 

