

module find_extrema 

	//////////////////PARAMETERS///////////////////////////

#(
	parameter 
	DATA_SIZE_BITS			 = 16, 
	PRECISION_BITS_MEMORY = 8, 
	CHECKING_PRECISION 	 = 8
)

	//////////////////INPUTS AND OUTPUTS////////////////////

(
	input 															 clk,
	input  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in,
	input 															 data_in_available,
	input 															 first,
	input 															 last,
	output [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out,
	output 															 point_is_max,
	output 															 point_is_min,
	output reg 														 data_out_available,
	output reg 														 first_out,
	output reg														 last_out,
	input																 reset
);

	//////////////////REGISTERS/////////////////////////////

	reg 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out_general;
	reg 																 point_is_max_general;
	reg 																 point_is_min_general;
	reg 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in_one;
	reg 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in_two;
	reg 	 [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in_three;
	reg 																 first_reg_one;
	reg																 first_reg_two;
	reg																 last_reg_one;
	reg																 last_reg_two;
	reg																 updated;
	reg																 running;
				//Equal Handling
	reg x_extreme_higher_reg;
	reg x_extreme_lower_reg;

	//////////////////WIRES////////////////////////////////

	wire  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out_equal;
	wire 																x_extreme_higher;
	wire 																x_extreme_lower;
	wire																higher_extreme_x;
	wire 																lower_extreme_x;
	wire 																equal_equal_x;
	wire 																point_is_max_equal;
	wire 																point_is_min_equal;

	//////////////////COMBINATORIAL////////////////////////

	assign x_extreme_higher =   ($signed(data_in_two[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])<$signed(data_in_one[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])&&running);
	assign x_extreme_lower  =   ($signed(data_in_two[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])>$signed(data_in_one[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])&&running);
	assign lower_extreme_x  = ($signed(data_in_three[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])<$signed(data_in_two[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])&&running);
	assign higher_extreme_x = ($signed(data_in_three[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])>$signed(data_in_two[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])&&running); 

							//OUTPUT

	assign point_is_max = point_is_max_equal||point_is_max_general;
	assign point_is_min = point_is_min_equal||point_is_min_general;
	assign data_out = data_out_equal | data_out_general;

							//Equal handling

	assign equal_equal_x = ($signed(data_in_three[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION]) == $signed(data_in_two[DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:PRECISION_BITS_MEMORY - CHECKING_PRECISION])&running);
	assign point_is_min_equal = higher_extreme_x&& x_extreme_higher_reg;
	assign point_is_max_equal = lower_extreme_x&&x_extreme_lower_reg; 
	assign data_out_equal = (point_is_max_equal||point_is_min_equal) ? data_in_two :0;


	always @* begin
		if(reset) begin
			data_out_available 	<= 1'b0;
			data_out_general		<= 0;
			point_is_max_general	<= 1'b0;
			point_is_min_general	<= 1'b0;	
			first_out 			<= 0;
			last_out 			<= 0; 
		end
		else if (first_reg_two&&updated) begin
			data_out_general	<= data_in_two;
			data_out_available 	<= 1;
			point_is_max_general	<= 1'b1;
			point_is_min_general	<= 1'b1;
			first_out			<= 1;
			last_out			<= 0;
		end
		else if(last_reg_two) begin
			if (updated) begin 
				data_out_general		<= data_in_two;
			end
			else begin 
				data_out_general		<= data_in_three;
			end
			data_out_available 	<= 1;
			point_is_max_general	<= 1'b1;
			point_is_min_general	<= 1'b1;
			first_out			<= 0;
			last_out			<= 1;
		end
		else if(updated && running) begin 
			data_out_available 	<= 1'b1;
			first_out			<= 0;
			last_out			<= 0;
			if (lower_extreme_x && x_extreme_lower) begin
				data_out_general		<= data_in_two;
				point_is_max_general	<= 1'b1;
				point_is_min_general <= 1'b0;
			end

			else if(higher_extreme_x && x_extreme_higher) begin
				data_out_general 		<= data_in_two;
				point_is_max_general	<= 1'b0;
				point_is_min_general	<= 1'b1;
			end

			else begin 
				data_out_general		<= 0;
				point_is_max_general	<= 1'b0;
				point_is_min_general	<= 1'b0;
			end
		end
		else begin
			data_out_available 	<= 1'b0;
			data_out_general		<= 0;
			point_is_max_general	<= 1'b0;
			point_is_min_general	<= 1'b0;	
			first_out			<= 0;
			last_out			<= 0;
		end
	end


	//////////////////SEQUENTIAL///////////////////////////

	always @(posedge clk or posedge reset) begin
		if(reset) begin
		updated			<= 0;
		data_in_three 	<= 0;
		data_in_two 	<= 0;
		data_in_one 	<= 0;
		first_reg_one 	<= 0;
		first_reg_two 	<= 0;
		last_reg_one 	<= 0;
		last_reg_two 	<= 0;
		end
		else begin
			updated				<= data_in_available;
			last_reg_one 		<= last;
			last_reg_two 		<= last_reg_one;
			if(data_in_available) begin
				data_in_three 	<= data_in;
				data_in_two 	<= data_in_three;
				data_in_one 	<= data_in_two;
				first_reg_one 	<= first;
				first_reg_two 	<= first_reg_one;		
			end 
		end
	end


	always @(posedge clk or posedge reset) begin
		if(reset)
			running <= 0;
		else if (first_reg_two & data_in_available)
			running 		<= 1;
		else if(last_reg_two)
			running 		<= 0;
	end


	always@(posedge clk or posedge reset) begin
		if(reset) begin
			x_extreme_higher_reg <=0;
			x_extreme_lower_reg <= 0; 
		end
		else if(equal_equal_x&&x_extreme_higher) begin
			x_extreme_higher_reg <= 1;
		end
		else if(equal_equal_x&&x_extreme_lower) begin
			x_extreme_lower_reg <= 1; 
		end
		else if(!equal_equal_x) begin 
			x_extreme_higher_reg <=0;
			x_extreme_lower_reg <= 0; 
		end 
	end 
endmodule
