

module sift 

	//////////////////PARAMETERS///////////////////////

#(
	parameter 
	DATA_SIZE_BITS        = 16, 
	PRECISION_BITS_CS 	 = 16, 
	PRECISION_BITS_MEMORY = 8, 
	CHECKING_PRECISION 	 = 4, 
	DEPTH 					 = 512
)

	//////////////////INPUTS AND OUTPUTS//////////////

(
	input 															 clk,
	input  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in,
	input 															 data_in_available,
	input 															 first,
	input 															 last,
	output [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] IMF_out,
	output															 IMF_out_available,
	input																 reset
);

	//////////////////REGISTERS///////////////////////

	reg 																read_out;

	//////////////////WIRES///////////////////////////

	wire 																first_out_controller;
	wire 																last_out_controller;
	wire 																first_in_controller;
	wire 																last_in_controller;
	wire 																data_out_controller_available;
	wire 																data_out_spline_available;
	wire 																first_out_spline;
	wire 																last_out_spline;
	wire  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out_spline;
	wire  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_in_controller;
	wire  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out_controller;

	//////////////////MODULES/////////////////////////
	 
	AVG_CS_CONTROLLER	#(
							  DATA_SIZE_BITS, 
							  PRECISION_BITS_MEMORY, 
							  CHECKING_PRECISION, 
							  DEPTH
							 )
							  
							  controller
							  
							 (
							  clk, 
							  !reset, 
							  data_in_controller, 
							  data_out_spline_available, 
							  data_in_available, 
							  first_in_controller, 
							  last_in_controller, 
							  data_out_controller, 
							  data_out_controller_available, 
							  first_out_controller,
							  last_out_controller, 
							  IMF_out_available
							 );
							
								
	find_avg_spline 	#(
							  DATA_SIZE_BITS,
							  PRECISION_BITS_CS, 
							  PRECISION_BITS_MEMORY, 
							  CHECKING_PRECISION, 
							  DEPTH
							 )
							  
							  avg_spline
							  
							 (
							  clk, 
							  data_out_controller, 
							  data_out_controller_available, 
							  first_out_controller, 
							  last_out_controller, 
							  data_out_spline, 
							  first_out_spline, 
							  last_out_spline, 
							  data_out_spline_available, 
							  (reset || IMF_out_available)
							 );
							  
							  
	//////////////////COMBINATORIAL///////////////////

	assign IMF_out 				= data_out_controller;
	assign data_in_controller  = (data_in_available == 1) ? data_in : data_out_spline;
	assign first_in_controller = (data_in_available == 1) ? first   : first_out_spline;
	assign last_in_controller  = (data_in_available == 1) ? last    : last_out_spline;

endmodule 