% Performs EMD with floating point precision, reads the fixed point
% precision results generated by ModelSim, displays the difference between
% the fixed point, and floating point results, and plots the signal
% to noise ratio (SNR). The test signal is a specified number of uniformly
% distributed noise samples. 
%
% inputs:
%               samples: Number of samples to generat for the test data
%                        set (must be a power of 2).
% precision_bits_memory: How many precision bits used for temporary RAM
%                        storage, specified in EMD_tb.v.
%
% How to use: 
% 1) Make sure the number of samples, and precision bits memory are the
%    same as in EMD_tb.v.
% 2) When "Press Enter to Continue... " appears in the command window,
%    run the testbench with RTL simulation in ModelSim, and wait for it to
%    finish. Make sure the "EMD_finished" signal has become high. If not,
%    run the simulation until it does.
% 3) Continue this script by pressing "Enter"

function compare_IMF_results(samples, precision_bits_memory)
    
    close all
    signal = rand(1,samples)-0.5;
    %load('x_test.mat')
    %time = 1;
    %fs_max = 0.2;
    %fs_min = 0;
    %freq_range = fs_max - fs_min;
    %Fs = samples/time; 
    %f_max = fs_max*Fs;
    %f_min = fs_min*Fs;
    %phase = 2*pi;

    %signal = zeros(1,samples);

    %for i = 1:2000
    %signal  =  signal + sine_gen(rand(),(rand()*freq_range+fs_min)*Fs,(rand()-0.5)*phase,time,samples);
    %end

    signal = round(signal/max(signal)*(2^13));
    figure('Name','Test data set','NumberTitle','off');
    plot(signal)
    xlabel('n')
    ylabel('Amplitude')
    title('Test data set')
    xlim([0 samples])
    legend('x_{test}[n]')

    generate_quartus_test_data_file(signal, samples);

    fprintf('Finished generating "test_data.dat" \n\n')
    fprintf('Run simulation in ModelSim to get the fixed point precision result data \n\n')
    disp('Press Enter to Continue...')
    pause;
    S = plot_and_compare(signal, samples, precision_bits_memory);
    figure('Name','SNR for each IMF_k','NumberTitle','off');
    plot(10*log(S))
    title('SNR for each IMF_{k}')
    xlabel('k')
    ylabel('SNR [dB]')
    legend('SNR[k]')   
    
end

