%% Wikipedia
% ai*xi-1+ bi*xi + ci*xi-1 = di
% https://en.wikipedia.org/wiki/Tridiagonal_matrix_algorithm

function [y_dd,h] =  find_second_derivative_tri_mat_alg(x,y)
    assert(length(x) == length(y), 'x and y must be at the same length');
    assert(length(x)>=3, 'dimention must be greater or equal to 3');
    nr_points = length(x);

    %% initializing   
    h = zeros(1,nr_points-1);
    y_dd = zeros(1,nr_points);
    A = zeros(1,nr_points);
    B = zeros(1, nr_points);
    C = zeros(1,nr_points);
    D = zeros(1, nr_points);
    C_prime = zeros(1, nr_points);
    D_prime = zeros(1, nr_points);
    A(nr_points)= 0;
    B(nr_points)= 1;
    C(nr_points)= 0;
    D(nr_points)= 0;
    A(1)= 0;
    B(1)= 1;    
    C(1)= 0;
    D(1)= 0;
    C_prime(1) = 0;
    D_prime(1) = 0;

    beta_denominator = zeros(1,nr_points-2);
    tmp = zeros(1,nr_points-2);
    tmp_2 = zeros(1,nr_points-2);
%% find h
    for i = 1:nr_points-1
        h(i) = x(i+1)-x(i);
    end 

%% Calculate coefficiants    
    for k = 2:nr_points-1
        %% Normal
        A(k)= h(k-1);
        B(k)= 2*(h(k-1)+h(k));
        C(k)= h(k);
        D(k)= 6*((y(k+1)-y(k))/h(k) - (y(k)- y(k-1))/h(k-1));
        %% Redused division
%         A(k)= h(k)*h(k-1)^2; 
%         B(k)= 2*(h(k-1)^2*h(k)+h(k)^2*h(k-1));
%         C(k)= h(k-1)*h(k)^2;
%         D(k)= 6*((y(k+1)-y(k))*h(k-1) - (y(k)- y(k-1))*h(k)); 
        

    end 
    
%% Calculate prime
    for i = 2:nr_points-1
        C_prime(i) = C(i)/(B(i)-A(i)*C_prime(i-1));
        D_prime(i) =  (D(i)-A(i)*D_prime(i-1))/(B(i)-A(i)*C_prime(i-1));
    end
    i = nr_points;
    D_prime(i) =  (D(i)-A(i)*D_prime(i-1))/(B(i)-A(i)*C_prime(i-1));   
    
%% Calculate second derivatives
    y_dd(nr_points) = D_prime(nr_points);
    i = 0;
    for i = nr_points-1:-1:1
        y_dd(i) = D_prime(i)-C_prime(i)*y_dd(i+1);
    end
end

    